@@ -134,7 +134,7 @@ int settings_get_level(const char *key)
 	const char *str;
 
 	str = settings_get_str_type(key, SETTING_TYPE_LEVEL);
-	return str == NULL ? 0 : level2bits(str);
+	return str == NULL ? 0 : level2bits(str, NULL);
 }
 
 int settings_get_size(const char *key)
@@ -355,6 +355,12 @@ int settings_set_time(const char *key, const char *value)
 
 int settings_set_level(const char *key, const char *value)
 {
+	int iserror;
+
+	(void)level2bits(value, &iserror);
+	if (iserror)
+		return FALSE;
+
         iconfig_node_set_str(settings_get_node(key), key, value);
 	return TRUE;
 }
@@ -453,9 +459,9 @@ static int backwards_compatibility(const char *module, CONFIG_NODE *node,
 
 	/* fe-text term_type -> fe-common/core term_charset - for 0.8.10-> */
 	if (strcmp(module, "fe-text") == 0) {
-		if (strcasecmp(node->key, "term_type") == 0 ||
+		if (g_ascii_strcasecmp(node->key, "term_type") == 0 ||
 		    /* kludge for cvs-version where term_charset was in fe-text */
-		    strcasecmp(node->key, "term_charset") == 0) {
+		    g_ascii_strcasecmp(node->key, "term_charset") == 0) {
 			new_module = "fe-common/core";
 			new_key = "term_charset";
 			new_value = !is_valid_charset(node->value) ? NULL :
@@ -472,8 +478,8 @@ static int backwards_compatibility(const char *module, CONFIG_NODE *node,
 			g_free(new_value);
 			config_changed = TRUE;
 			return new_key != NULL;
-		} else if (strcasecmp(node->key, "actlist_moves") == 0 &&
-			   node->value != NULL && strcasecmp(node->value, "yes") == 0) {
+		} else if (g_ascii_strcasecmp(node->key, "actlist_moves") == 0 &&
+			   node->value != NULL && g_ascii_strcasecmp(node->value, "yes") == 0) {
 			config_node_set_str(mainconfig, parent, "actlist_sort", "recent");
 			config_node_set_str(mainconfig, parent, node->key, NULL);
 			config_changed = TRUE;
@@ -500,7 +506,7 @@ void settings_check_module(const char *module)
 	if (node == NULL) return;
 
         errors = g_string_new(NULL);
-	g_string_sprintf(errors, "Unknown settings in configuration "
+	g_string_printf(errors, "Unknown settings in configuration "
 			 "file for module %s:", module);
 
         count = 0;
@@ -515,7 +521,7 @@ void settings_check_module(const char *module)
 			continue;
 
 		if (set == NULL || strcmp(set->module, module) != 0) {
-			g_string_sprintfa(errors, " %s", node->key);
+			g_string_append_printf(errors, " %s", node->key);
                         count++;
 		}
 	}
@@ -542,7 +548,10 @@ void settings_check_module(const char *module)
 
 static int settings_compare(SETTINGS_REC *v1, SETTINGS_REC *v2)
 {
-	return strcmp(v1->section, v2->section);
+	int cmp = strcmp(v1->section, v2->section);
+	if (!cmp)
+		cmp = strcmp(v1->key, v2->key);
+	return cmp;
 }
 
 static void settings_hash_get(const char *key, SETTINGS_REC *rec,
