@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -24,6 +24,7 @@
 #include "commands.h"
 #include "pidwait.h"
 #include "line-split.h"
+#include "network.h"
 #include "net-sendbuffer.h"
 #include "misc.h"
 #include "levels.h"
@@ -209,17 +210,17 @@ static int signal_name_to_id(const char *name)
 	/* check only the few most common signals, too much job to check
 	   them all. if we sometimes want more, procps-sources/proc/sig.c
 	   would be useful for copypasting */
-	if (g_strcasecmp(name, "hup") == 0)
+	if (g_ascii_strcasecmp(name, "hup") == 0)
                 return SIGHUP;
-	if (g_strcasecmp(name, "int") == 0)
+	if (g_ascii_strcasecmp(name, "int") == 0)
                 return SIGINT;
-	if (g_strcasecmp(name, "term") == 0)
+	if (g_ascii_strcasecmp(name, "term") == 0)
                 return SIGTERM;
-	if (g_strcasecmp(name, "kill") == 0)
+	if (g_ascii_strcasecmp(name, "kill") == 0)
                 return SIGKILL;
-	if (g_strcasecmp(name, "usr1") == 0)
+	if (g_ascii_strcasecmp(name, "usr1") == 0)
                 return SIGUSR1;
-	if (g_strcasecmp(name, "usr2") == 0)
+	if (g_ascii_strcasecmp(name, "usr2") == 0)
                 return SIGUSR2;
         return -1;
 }
@@ -349,17 +350,12 @@ static void process_exec(PROCESS_REC *rec, const char *cmd)
 static void sig_exec_input_reader(PROCESS_REC *rec)
 {
         char tmpbuf[512], *str;
-        gsize recvlen;
-	int err, ret;
+        int recvlen;
+	int ret;
 
 	g_return_if_fail(rec != NULL);
 
-	recvlen = 0;
-	err = g_io_channel_read(rec->in, tmpbuf,
-				sizeof(tmpbuf), &recvlen);
-	if (err != 0 && err != G_IO_ERROR_AGAIN && errno != EINTR)
-		recvlen = -1;
-
+	recvlen = net_receive(rec->in, tmpbuf, sizeof(tmpbuf));
 	do {
 		ret = line_split(tmpbuf, recvlen, &str, &rec->databuf);
 		if (ret == -1) {
@@ -517,7 +513,7 @@ static void handle_exec(const char *args, GHashTable *optlist,
 	rec->name = g_strdup(g_hash_table_lookup(optlist, "name"));
 
 	level = g_hash_table_lookup(optlist, "level");
-	rec->level = level == NULL ? MSGLEVEL_CLIENTCRAP : level2bits(level);
+	rec->level = level == NULL ? MSGLEVEL_CLIENTCRAP : level2bits(level, NULL);
 
 	rec->read_tag = g_input_add(rec->in, G_INPUT_READ,
 				    (GInputFunction) sig_exec_input_reader,
