@@ -39,7 +39,6 @@
 #include "gui-readline.h"
 #include "statusbar.h"
 #include "gui-windows.h"
-#include "textbuffer-reformat.h"
 #include "irssi-version.h"
 
 #include <signal.h>
@@ -86,7 +85,7 @@ static const char *firsttimer_text =
 	"Looks like this is the first time you've run irssi.\n"
 	"This is just a reminder that you really should go read\n"
 	"startup-HOWTO if you haven't already. You can find it\n"
-	"and more irssi beginner info at http://irssi.org/help/\n"
+	"and more irssi beginner info at http://www.irssi.org\n"
 	"\n"
 	"For the truly impatient people who don't like any automatic\n"
 	"window creation or closing, just type: /MANUAL-WINDOWS";
@@ -168,7 +167,6 @@ static void textui_finish_init(void)
 	        textbuffer_init();
 	        textbuffer_view_init();
 		textbuffer_commands_init();
-		textbuffer_reformat_init();
 		gui_expandos_init();
 		gui_printtext_init();
 		gui_readline_init();
@@ -230,7 +228,6 @@ static void textui_deinit(void)
 		mainwindow_activity_deinit();
 		mainwindows_deinit();
 		gui_expandos_deinit();
-		textbuffer_reformat_deinit();
 		textbuffer_commands_deinit();
 	        textbuffer_view_deinit();
 	        textbuffer_deinit();
@@ -247,42 +244,6 @@ static void textui_deinit(void)
 	core_deinit();
 }
 
-static void check_oldcrap(void)
-{
-        FILE *f;
-	char *path, str[256];
-        int found;
-
-        /* check that default.theme is up-to-date */
-	path = g_strdup_printf("%s/default.theme", get_irssi_dir());
-	f = fopen(path, "r+");
-	if (f == NULL) {
-		g_free(path);
-                return;
-	}
-        found = FALSE;
-	while (!found && fgets(str, sizeof(str), f) != NULL)
-                found = strstr(str, "abstracts = ") != NULL;
-	fclose(f);
-
-	if (found) {
-		g_free(path);
-		return;
-	}
-
-	printf("\nYou seem to have an old default.theme in %s/ directory.\n", get_irssi_dir());
-        printf("Themeing system has changed a bit since last irssi release,\n");
-        printf("you should either delete your old default.theme or manually\n");
-        printf("merge it with the new default.theme.\n\n");
-	printf("Do you want to delete the old theme now? (Y/n)\n");
-
-	str[0] = '\0';
-	fgets(str, sizeof(str), stdin);
-	if (i_toupper(str[0]) == 'Y' || str[0] == '\n' || str[0] == '\0')
-                remove(path);
-	g_free(path);
-}
-
 static void check_files(void)
 {
 	struct stat statbuf;
@@ -290,8 +251,6 @@ static void check_files(void)
 	if (stat(get_irssi_dir(), &statbuf) != 0) {
 		/* ~/.irssi doesn't exist, first time running irssi */
 		display_firsttimer = TRUE;
-	} else {
-                check_oldcrap();
 	}
 }
 
@@ -329,12 +288,16 @@ GMemVTable gc_mem_table = {
 int main(int argc, char **argv)
 {
 	static int version = 0;
-	static struct poptOption options[] = {
-		{ "dummy", 'd', POPT_ARG_NONE, &dummy, 0, "Use the dummy terminal mode", NULL },
-		{ "version", 'v', POPT_ARG_NONE, &version, 0, "Display irssi version", NULL },
-		{ NULL, '\0', 0, NULL }
+	static GOptionEntry options[] = {
+		{ "dummy", 'd', 0, G_OPTION_ARG_NONE, &dummy, "Use the dummy terminal mode", NULL },
+		{ "version", 'v', 0, G_OPTION_ARG_NONE, &version, "Display irssi version", NULL },
+		{ NULL }
 	};
 
+#ifdef USE_GC
+	g_mem_set_vtable(&gc_mem_table);
+#endif
+
 	core_register_options();
 	fe_common_core_register_options();
 	args_register(options);
@@ -346,10 +309,6 @@ int main(int argc, char **argv)
 		return 0;
 	}
 
-#ifdef USE_GC
-	g_mem_set_vtable(&gc_mem_table);
-#endif
-
 	srand(time(NULL));
 
 	dummy = FALSE;
