@@ -20,9 +20,7 @@
 
 #include "module.h"
 
-#include "net-nonblock.h"
 #include "net-sendbuffer.h"
-#include "line-split.h"
 #include "signals.h"
 #include "rawlog.h"
 #include "misc.h"
@@ -41,6 +39,7 @@
 #include "modes.h"
 
 #include "settings.h"
+#include "recode.h"
 
 #define DEFAULT_MAX_KICKS 1
 #define DEFAULT_MAX_MODES 3
@@ -81,6 +80,7 @@ static void send_message(SERVER_REC *server, const char *target,
 	IRC_SERVER_REC *ircserver;
 	CHANNEL_REC *channel;
 	char *str;
+	char *recoded;
 
         ircserver = IRC_SERVER(server);
 	g_return_if_fail(ircserver != NULL);
@@ -94,9 +94,11 @@ static void send_message(SERVER_REC *server, const char *target,
 			target = channel->name;
 	}
 
-	str = g_strdup_printf("PRIVMSG %s :%s", target, msg);
+	recoded = recode_out(SERVER(server), msg, target);
+	str = g_strdup_printf("PRIVMSG %s :%s", target, recoded);
 	irc_send_cmd_split(ircserver, str, 2, ircserver->max_msgs_in_cmd);
 	g_free(str);
+	g_free(recoded);
 }
 
 static void server_init(IRC_SERVER_REC *server)
@@ -259,7 +261,7 @@ void irc_server_purge_output(IRC_SERVER_REC *server, const char *target)
                 redirect = tmp->next->data;
 
 		if ((target == NULL || command_has_target(cmd, target)) &&
-		    g_strncasecmp(cmd, "PONG ", 5) != 0) {
+		    g_ascii_strncasecmp(cmd, "PONG ", 5) != 0) {
                         /* remove the redirection */
                         link = tmp->next;
 			server->cmdqueue =
@@ -334,14 +336,44 @@ static void sig_disconnected(IRC_SERVER_REC *server)
 static void sig_server_quit(IRC_SERVER_REC *server, const char *msg)
 {
 	char *str;
+	char *recoded;
 
-	if (!IS_IRC_SERVER(server) || server->handle == NULL ||
-	    server->buffer == NULL)
+	if (!IS_IRC_SERVER(server) || !server->connected)
 		return;
 
-	str = g_strdup_printf("QUIT :%s", msg);
+	recoded = recode_out(SERVER(server), msg, NULL);
+	str = g_strdup_printf("QUIT :%s", recoded);
 	irc_send_cmd_now(server, str);
 	g_free(str);
+	g_free(recoded);
+}
+
+void irc_server_send_action(IRC_SERVER_REC *server, const char *target, const char *data)
+{
+	char *recoded;
+
+	recoded = recode_out(SERVER(server), data, target);
+	irc_send_cmdv(server, "PRIVMSG %s :\001ACTION %s\001", target, recoded);
+	g_free(recoded);
+}
+
+void irc_server_send_away(IRC_SERVER_REC *server, const char *reason)
+{
+	char *recoded = NULL;
+
+	if (!IS_IRC_SERVER(server))
+		return;
+
+	if (*reason != '\0' || server->usermode_away) {
+		g_free_and_null(server->away_reason);
+                if (*reason != '\0') {
+			server->away_reason = g_strdup(reason);
+			reason = recoded = recode_out(SERVER(server), reason, NULL);
+		}
+
+		irc_send_cmdv(server, "AWAY :%s", reason);
+	}
+	g_free(recoded);
 }
 
 void irc_server_send_data(IRC_SERVER_REC *server, const char *data, int len)
@@ -366,7 +398,7 @@ void irc_server_send_data(IRC_SERVER_REC *server, const char *data, int len)
 	}
 }
 
-static void server_cmd_timeout(IRC_SERVER_REC *server, GTimeVal *now)
+static int server_cmd_timeout(IRC_SERVER_REC *server, GTimeVal *now)
 {
 	REDIRECT_REC *redirect;
         GSList *link;
@@ -375,20 +407,20 @@ static void server_cmd_timeout(IRC_SERVER_REC *server, GTimeVal *now)
 	int len;
 
 	if (!IS_IRC_SERVER(server))
-		return;
+		return 0;
 
 	if (server->cmdcount == 0 && server->cmdqueue == NULL)
-		return;
+		return 0;
 
 	if (g_timeval_cmp(now, &server->wait_cmd) == -1)
-		return;
+		return 1;
 
 	usecs = get_timeval_diff(now, &server->last_cmd);
 	if (usecs < server->cmd_queue_speed)
-		return;
+		return 1;
 
 	server->cmdcount--;
-	if (server->cmdqueue == NULL) return;
+	if (server->cmdqueue == NULL) return 1;
 
         /* get command */
 	cmd = server->cmdqueue->data;
@@ -398,8 +430,7 @@ static void server_cmd_timeout(IRC_SERVER_REC *server, GTimeVal *now)
 	len = strlen(cmd);
 	irc_server_send_data(server, cmd, len);
 
-	/* add to rawlog without [CR+]LF (/RAWQUOTE might not have
-	   added the CR) */
+	/* add to rawlog without [CR+]LF */
         if (len > 2 && cmd[len-2] == '\r')
 		cmd[len-2] = '\0';
         else if (cmd[len-1] == '\n')
@@ -414,16 +445,33 @@ static void server_cmd_timeout(IRC_SERVER_REC *server, GTimeVal *now)
         link = server->cmdqueue;
 	server->cmdqueue = g_slist_remove_link(server->cmdqueue, link);
         g_slist_free_1(link);
+	return 1;
 }
 
 /* check every now and then if there's data to be sent in command buffer */
 static int servers_cmd_timeout(void)
 {
 	GTimeVal now;
+	GSList *tmp;
+	int keep = 0;
 
 	g_get_current_time(&now);
-	g_slist_foreach(servers, (GFunc) server_cmd_timeout, &now);
-	return 1;
+	for (tmp = servers; tmp != NULL; tmp = tmp->next) {
+		keep |= server_cmd_timeout(tmp->data, &now);
+	}
+	if (keep)
+		return 1;
+	else {
+		cmd_tag = -1;
+		return 0;
+	}
+}
+
+/* Start the timeout for sending data later and decreasing cmdcount again */
+void irc_servers_start_cmd_timeout(void)
+{
+	if (cmd_tag == -1)
+		cmd_tag = g_timeout_add(500, (GSourceFunc) servers_cmd_timeout, NULL);
 }
 
 /* Return a string of all channels (and keys, if any have them) in server,
@@ -445,8 +493,8 @@ char *irc_server_get_channels(IRC_SERVER_REC *server)
 	for (tmp = server->channels; tmp != NULL; tmp = tmp->next) {
 		CHANNEL_REC *channel = tmp->data;
 
-		g_string_sprintfa(chans, "%s,", channel->name);
-		g_string_sprintfa(keys, "%s,", channel->key == NULL ? "x" :
+		g_string_append_printf(chans, "%s,", channel->name);
+		g_string_append_printf(keys, "%s,", channel->key == NULL ? "x" :
 				  channel->key);
 		if (channel->key != NULL)
 			use_keys = TRUE;
@@ -456,8 +504,8 @@ char *irc_server_get_channels(IRC_SERVER_REC *server)
 	for (tmp = server->rejoin_channels; tmp != NULL; tmp = tmp->next) {
 		REJOIN_REC *rec = tmp->data;
 
-		g_string_sprintfa(chans, "%s,", rec->channel);
-		g_string_sprintfa(keys, "%s,", rec->key == NULL ? "x" :
+		g_string_append_printf(chans, "%s,", rec->channel);
+		g_string_append_printf(keys, "%s,", rec->key == NULL ? "x" :
 				  rec->key);
 		if (rec->key != NULL) use_keys = TRUE;
 	}
@@ -465,7 +513,7 @@ char *irc_server_get_channels(IRC_SERVER_REC *server)
 	if (chans->len > 0) {
 		g_string_truncate(chans, chans->len-1);
 		g_string_truncate(keys, keys->len-1);
-		if (use_keys) g_string_sprintfa(chans, " %s", keys->str);
+		if (use_keys) g_string_append_printf(chans, " %s", keys->str);
 	}
 
 	ret = chans->str;
@@ -475,34 +523,6 @@ char *irc_server_get_channels(IRC_SERVER_REC *server)
 	return ret;
 }
 
-static int sig_set_user_mode(IRC_SERVER_REC *server)
-{
-	const char *mode;
-	char *newmode, *args;
-
-	if (!IS_IRC_SERVER(server) || g_slist_find(servers, server) == NULL)
-		return 0; /* not an irc server or got disconnected */
-
-	mode = server->connrec->usermode;
-	newmode = server->usermode == NULL ? NULL :
-		modes_join(NULL, server->usermode, mode, FALSE);
-
-	if (newmode == NULL || strcmp(newmode, server->usermode) != 0) {
-		/* change the user mode. we used to do some trickery to
-		   get rid of unwanted modes at reconnect time, but that's
-		   more trouble than worth. (eg. we don't want to remove
-		   some good default server modes, but we don't want to
-		   set back +r, etc..) */
-		args = g_strdup_printf((*mode == '+' || *mode == '-') ? "%s %s" : 
-				       "%s +%s", server->nick, mode);
-		signal_emit("command mode", 3, args, server, NULL);
-		g_free(args);
-	}
-
-	g_free_not_null(newmode);
-	return 0;
-}
-
 static void event_connected(IRC_SERVER_REC *server, const char *data, const char *from)
 {
 	char *params, *nick;
@@ -533,8 +553,15 @@ static void event_connected(IRC_SERVER_REC *server, const char *data, const char
 	memcpy(&server->wait_cmd, &now, sizeof(GTimeVal));
 
 	if (server->connrec->usermode != NULL) {
-		/* wait a second and then send the user mode */
-		g_timeout_add(1000, (GSourceFunc) sig_set_user_mode, server);
+		/* Send the user mode, before the autosendcmd.
+		 * Do not pass this through cmd_mode because it
+		 * is not known whether the resulting MODE message
+		 * (if any) is the initial umode or a reply to this.
+		 */
+		irc_send_cmdv(server, "MODE %s %s", server->nick,
+				server->connrec->usermode);
+		g_free_not_null(server->wanted_usermode);
+		server->wanted_usermode = g_strdup(server->connrec->usermode);
 	}
 
 	signal_emit("event connected", 1, server);
@@ -807,6 +834,32 @@ void irc_server_init_isupport(IRC_SERVER_REC *server)
 		else
 			server->nick_comp_func = irc_nickcmp_ascii;
 	}
+
+	if ((sptr = g_hash_table_lookup(server->isupport, "TARGMAX"))) {
+		char *p = sptr;
+		server->max_kicks_in_cmd = 1;
+		server->max_msgs_in_cmd = 1;
+		/* Not doing WHOIS here until it is clear what it means. */
+		while (*p != '\0') {
+			if (!g_ascii_strncasecmp(p, "KICK:", 5)) {
+				server->max_kicks_in_cmd = atoi(p + 5);
+				if (server->max_kicks_in_cmd <= 0)
+					server->max_kicks_in_cmd = 30;
+			} else if (!g_ascii_strncasecmp(p, "PRIVMSG:", 8)) {
+				server->max_msgs_in_cmd = atoi(p + 8);
+				if (server->max_msgs_in_cmd <= 0)
+					server->max_msgs_in_cmd = 30;
+			}
+			p = strchr(p, ',');
+			if (p == NULL)
+				break;
+			p++;
+		}
+	} else if ((sptr = g_hash_table_lookup(server->isupport, "MAXTARGETS"))) {
+		server->max_msgs_in_cmd = atoi(sptr);
+		if (server->max_msgs_in_cmd <= 0)
+			server->max_msgs_in_cmd = 1;
+	}
 }
 
 void irc_servers_init(void)
@@ -815,7 +868,7 @@ void irc_servers_init(void)
 	settings_add_time("flood", "cmd_queue_speed", DEFAULT_CMD_QUEUE_SPEED);
 	settings_add_int("flood", "cmds_max_at_once", DEFAULT_CMDS_MAX_AT_ONCE);
 
-	cmd_tag = g_timeout_add(500, (GSourceFunc) servers_cmd_timeout, NULL);
+	cmd_tag = -1;
 
 	signal_add_first("server connected", (SIGNAL_FUNC) sig_connected);
 	signal_add_last("server disconnected", (SIGNAL_FUNC) sig_disconnected);
@@ -841,7 +894,8 @@ void irc_servers_init(void)
 
 void irc_servers_deinit(void)
 {
-	g_source_remove(cmd_tag);
+	if (cmd_tag != -1)
+		g_source_remove(cmd_tag);
 
 	signal_remove("server connected", (SIGNAL_FUNC) sig_connected);
 	signal_remove("server disconnected", (SIGNAL_FUNC) sig_disconnected);
