@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -46,8 +46,12 @@ static void nick_mode_change(IRC_CHANNEL_REC *channel, const char *nick,
 	if (mode == '@') nickrec->op = type == '+';
 	else if (mode == '+') nickrec->voice = type == '+';
 	else if (mode == '%') nickrec->halfop = type == '+';
-	else if (channel->server->prefix[(unsigned char) mode] != '\0')
-		nickrec->other = (type == '+' ? mode : '\0');
+	if (channel->server->prefix[(unsigned char) mode] != '\0') {
+		if (type == '+')
+			prefix_add(nickrec->prefixes, mode, (SERVER_REC *) channel->server);
+		else
+			prefix_del(nickrec->prefixes, mode);
+	}
 
 	modestr[0] = mode; modestr[1] = '\0';
 	typestr[0] = type; typestr[1] = '\0';
@@ -55,6 +59,57 @@ static void nick_mode_change(IRC_CHANNEL_REC *channel, const char *nick,
 		    channel, nickrec, setby, modestr, typestr);
 }
 
+void prefix_add(char prefixes[MAX_USER_PREFIXES+1], char newprefix, SERVER_REC *server)
+{
+	const char *prefixlst;
+	char newprefixes[MAX_USER_PREFIXES+1]; /* to hold the new prefixes */
+	unsigned int newpos = 0; /* to hold our position in the new prefixes */
+	unsigned int oldpos = 0; /* to hold our position in the old prefixes */
+
+	prefixlst = server->get_nick_flags(server);
+
+	/* go through the possible prefixes, copy higher ones, and find this one's place
+	 * always leave room for the current prefix to be added, though.
+	 */
+	while (*prefixlst != '\0' && prefixes[oldpos] != '\0' &&
+			newpos < MAX_USER_PREFIXES - 1) {
+		if (prefixes[oldpos] == newprefix)
+			return; /* already inserted.  why are we here? */
+
+		if (*prefixlst == newprefix)
+			break; /* insert the new prefix here */
+		
+		if (*prefixlst == prefixes[oldpos]) {
+			/* this prefix is present.
+			 * the one we are inserting goes after it.
+			 * copy it over, and continue searching.
+			 */
+			newprefixes[newpos++] = prefixes[oldpos++];
+		}
+		prefixlst++;
+	}
+	
+	/* newpos is now the position in which we wish to insert the prefix */
+	newprefixes[newpos++] = newprefix;
+
+	/* finish copying the remaining prefixes */
+	while (prefixes[oldpos] != '\0' && newpos < MAX_USER_PREFIXES)
+		newprefixes[newpos++] = prefixes[oldpos++];
+
+	newprefixes[newpos] = '\0';
+
+	strcpy(prefixes, newprefixes);
+}
+
+void prefix_del(char prefixes[MAX_USER_PREFIXES+1], char oldprefix)
+{
+	char *todel;
+
+	todel = strchr(prefixes, oldprefix);
+	if (todel)
+		memmove(todel, todel+1, strlen(todel));
+}
+
 static int mode_is_set(const char *str, char mode)
 {
 	char *end, *pos;
@@ -88,14 +143,8 @@ static void mode_add_arg(GString *str, int pos, int updating, const char *arg)
                 g_string_erase(str, pos, (int) (p-str->str)-pos);
 	}
 
-	/* .. GLib shouldn't fail when inserting at the end of the string */
-	if (pos == str->len) {
-		g_string_append_c(str, ' ');
-		g_string_append(str, arg);
-	} else {
-		g_string_insert_c(str, pos, ' ');
-		g_string_insert(str, pos+1, arg);
-	}
+	g_string_insert_c(str, pos, ' ');
+	g_string_insert(str, pos+1, arg);
 }
 
 /* Add mode character to list sorted alphabetically */
@@ -251,8 +300,6 @@ void modes_type_prefix(IRC_CHANNEL_REC *channel, const char *setby,
 		       char type, char mode, char *arg, GString *newmode)
 {
 	int umode = (unsigned char) mode;
-	nick_mode_change(channel, arg, channel->server->modes[umode].prefix,
-			 type, setby);
 
 	if (g_strcasecmp(channel->server->nick, arg) == 0) {
 		/* see if we need to update channel->chanop */
@@ -274,6 +321,9 @@ void modes_type_prefix(IRC_CHANNEL_REC *channel, const char *setby,
 				channel->chanop = type == '+';
 		}
 	}
+
+	nick_mode_change(channel, arg, channel->server->modes[umode].prefix,
+			 type, setby);
 }
 
 int channel_mode_is_set(IRC_CHANNEL_REC *channel, char mode)
@@ -409,7 +459,7 @@ static void parse_user_mode(IRC_SERVER_REC *server, const char *modestr)
 	newmode = modes_join(NULL, server->usermode, modestr, FALSE);
 	oldmode = server->usermode;
 	server->usermode = newmode;
-	server->server_operator = (strchr(newmode, 'o') != NULL);
+	server->server_operator = ((strchr(newmode, 'o') != NULL) || (strchr(newmode, 'O') != NULL));
 
 	signal_emit("user mode changed", 2, server, oldmode);
 	g_free_not_null(oldmode);
@@ -519,7 +569,7 @@ void channel_set_singlemode(IRC_CHANNEL_REC *channel, const char *nicks,
 
 		if (num == 0)
 		{
-			g_string_sprintf(str, "MODE %s %s",
+			g_string_printf(str, "MODE %s %s",
 					 channel->name, mode);
 			modepos = str->len;
 		} else {
@@ -527,7 +577,7 @@ void channel_set_singlemode(IRC_CHANNEL_REC *channel, const char *nicks,
 			g_string_insert(str, modepos, mode);
 		}
 
-		g_string_sprintfa(str, " %s", *nick);
+		g_string_append_printf(str, " %s", *nick);
 
 		if (++num == channel->server->max_modes_in_cmd) {
 			/* max. modes / command reached, send to server */
@@ -609,7 +659,7 @@ void channel_set_mode(IRC_SERVER_REC *server, const char *channel,
 			}
 
 			if (*arg != '\0')
-				g_string_sprintfa(targs, " %s", arg);
+				g_string_append_printf(targs, " %s", arg);
 		}
 	}
 
@@ -645,7 +695,7 @@ static int get_wildcard_nicks(GString *output, const char *mask,
 		if (g_strcasecmp(rec->nick, channel->server->nick) == 0)
 			continue;
 
-		g_string_sprintfa(output, "%s ", rec->nick);
+		g_string_append_printf(output, "%s ", rec->nick);
                 count++;
 	}
 	g_slist_free(nicks);
@@ -684,7 +734,7 @@ static char *get_nicks(IRC_SERVER_REC *server, WI_ITEM_REC *item,
 		if (strchr(*match, '*') == NULL &&
 		    strchr(*match, '?') == NULL) {
 			/* no wildcards */
-                        g_string_sprintfa(str, "%s ", *match);
+                        g_string_append_printf(str, "%s ", *match);
 		} else {
 			count = get_wildcard_nicks(str, *match, channel,
 						   op, voice);
