@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -23,9 +23,7 @@
 #include "commands.h"
 #include "recode.h"
 #include "network.h"
-#include "net-nonblock.h"
 #include "net-sendbuffer.h"
-#include "line-split.h"
 #include "misc.h"
 #include "settings.h"
 
@@ -92,7 +90,6 @@ static void sig_dcc_destroyed(CHAT_DCC_REC *dcc)
 	dcc_remove_chat_refs(dcc);
 
 	if (dcc->sendbuf != NULL) net_sendbuffer_destroy(dcc->sendbuf, FALSE);
-	line_split_free(dcc->readbuf);
 	g_free(dcc->id);
 }
 
@@ -298,15 +295,14 @@ static void cmd_ctcp(const char *data, IRC_SERVER_REC *server)
 /* input function: DCC CHAT received some data.. */
 void dcc_chat_input(CHAT_DCC_REC *dcc)
 {
-        char tmpbuf[512], *str;
-	int recvlen, ret;
+	char *str;
+	int ret;
 
 	g_return_if_fail(IS_DCC_CHAT(dcc));
 
 	do {
-		recvlen = net_receive(dcc->handle, tmpbuf, sizeof(tmpbuf));
+		ret = net_sendbuffer_receive_line(dcc->sendbuf, &str, 1);
 
-		ret = line_split(tmpbuf, recvlen, &str, &dcc->readbuf);
 		if (ret == -1) {
 			/* connection lost */
                         dcc->connection_lost = TRUE;
@@ -540,7 +536,7 @@ static void cmd_mircdcc(const char *data, SERVER_REC *server,
 	if (dcc == NULL) return;
 
 	dcc->mirc_ctcp = i_toupper(*data) != 'N' &&
-		g_strncasecmp(data, "OF", 2) != 0;
+		g_ascii_strncasecmp(data, "OF", 2) != 0;
 }
 
 /* DCC CLOSE CHAT <nick> - check only from chat_ids in open DCC chats,
@@ -554,7 +550,7 @@ static void cmd_dcc_close(char *data, SERVER_REC *server)
 
 	g_return_if_fail(data != NULL);
 
-	if (g_strncasecmp(data, "CHAT ", 5) != 0 ||
+	if (g_ascii_strncasecmp(data, "CHAT ", 5) != 0 ||
 	    !cmd_get_params(data, &free_arg, 2, NULL, &nick))
 		return;
 
@@ -695,11 +691,11 @@ static void dcc_chat_msg(CHAT_DCC_REC *dcc, const char *msg)
 	g_return_if_fail(msg != NULL);
 
 	reply = FALSE;
-	if (g_strncasecmp(msg, "CTCP_MESSAGE ", 13) == 0) {
+	if (g_ascii_strncasecmp(msg, "CTCP_MESSAGE ", 13) == 0) {
 		/* bitchx (and ircii?) sends this */
 		msg += 13;
 		dcc->mirc_ctcp = FALSE;
-	} else if (g_strncasecmp(msg, "CTCP_REPLY ", 11) == 0) {
+	} else if (g_ascii_strncasecmp(msg, "CTCP_REPLY ", 11) == 0) {
 		/* bitchx (and ircii?) sends this */
 		msg += 11;
 		reply = TRUE;
@@ -721,8 +717,7 @@ static void dcc_chat_msg(CHAT_DCC_REC *dcc, const char *msg)
 	ptr = strchr(cmd, ' ');
 	if (ptr != NULL) *ptr++ = '\0'; else ptr = "";
 
-	cmd = g_strdup(cmd);
-	g_strup(cmd);
+	cmd = g_ascii_strup(cmd, -1);
 
 	g_strdown(event+9);
 	if (!signal_emit(event, 2, dcc, ptr)) {
@@ -763,7 +758,7 @@ static void ctcp_reply_dcc_reject(IRC_SERVER_REC *server, const char *data,
 
 	/* default REJECT handler checks args too -
 	   we don't care about it in DCC chats. */
-	if (g_strncasecmp(data, "CHAT", 4) == 0 &&
+	if (g_ascii_strncasecmp(data, "CHAT", 4) == 0 &&
 	    (data[4] == '\0' || data[4] == ' ')) {
 		dcc = dcc_find_request(DCC_CHAT_TYPE, nick, NULL);
 		if (dcc != NULL) dcc_close(dcc);
