@@ -22,7 +22,6 @@
 #include "signals.h"
 #include "commands.h"
 #include "network.h"
-#include "line-split.h"
 #include "misc.h"
 #include "settings.h"
 #include "ignore.h"
@@ -333,7 +332,7 @@ static void sig_server_nick_changed(IRC_SERVER_REC *server)
 static void ctcp_msg(IRC_SERVER_REC *server, const char *data,
 		     const char *nick, const char *addr, const char *target)
 {
-	if (g_strncasecmp(data, "dcc ", 4) != 0)
+	if (g_ascii_strncasecmp(data, "dcc ", 4) != 0)
                 return;
 	data += 4;
 
@@ -346,7 +345,7 @@ static void ctcp_msg(IRC_SERVER_REC *server, const char *data,
 static void ctcp_reply(IRC_SERVER_REC *server, const char *data,
 		       const char *nick, const char *addr, const char *target)
 {
-	if (g_strncasecmp(data, "dcc ", 4) != 0)
+	if (g_ascii_strncasecmp(data, "dcc ", 4) != 0)
                 return;
 	data += 4;
 
